@@ -63,6 +63,7 @@ struct _CDDisplayPrivate {
 enum {
 	PLAYMODE_CHANGED,
 	LOOPMODE_CHANGED,
+	REMAINING_TIME,
 	LAST_SIGNAL
 };
 
@@ -487,11 +488,23 @@ button_press_event (GtkWidget *widget,
 {
 	CDDisplay *disp = CD_DISPLAY (widget);
 	CDDisplayPrivate *priv = disp->priv;
-
+	GnomeCDText *text;
+	PangoRectangle rect;
+	
+	text = priv->layout[CD_DISPLAY_LINE_TIME];
+	pango_layout_get_extents (text->layout, NULL, &rect);
+	text->height = rect.height / 1000;
+	
 	if (ev->button != 1) {
 		return FALSE;
 	}
-	
+
+    /* Check the remaining time mode */
+    if (ev->x <= (rect.width / 1000) && ev->y <= text->height) {
+        g_signal_emit (G_OBJECT (widget), display_signals[REMAINING_TIME], 0, NULL);
+        return TRUE;
+	}
+
 	/* Check the play mode */
 	if (ev->x >= priv->track->rect.x &&
 	    ev->x <= (priv->track->rect.x + priv->track->rect.width) &&
@@ -553,6 +566,14 @@ class_init (CDDisplayClass *klass)
 							  g_cclosure_marshal_VOID__INT,
 							  G_TYPE_NONE,
 							  1, G_TYPE_INT);
+	display_signals[REMAINING_TIME] = g_signal_new ("remainingtime-mode-changed",
+							  G_TYPE_FROM_CLASS (klass),
+							  G_SIGNAL_RUN_FIRST | G_SIGNAL_NO_RECURSE,
+							  G_STRUCT_OFFSET (CDDisplayClass, remainingtime_mode_changed),
+							  NULL, NULL,
+							  g_cclosure_marshal_VOID__INT,
+							  G_TYPE_NONE,
+							  1, G_TYPE_INT);
 	display_signals[LOOPMODE_CHANGED] = g_signal_new ("loopmode-changed",
 							  G_TYPE_FROM_CLASS (klass),
 							  G_SIGNAL_RUN_FIRST | G_SIGNAL_NO_RECURSE,
@@ -783,14 +804,28 @@ cd_display_clear (CDDisplay *disp)
 	gtk_widget_queue_resize (GTK_WIDGET (disp));
 }
 
+/* given a theme name and a file name, create the full path to the image file */
 static inline char *
 make_fullname (const char *theme_name,
 	       const char *name)
 {
 	char *image;
+	char *tmp;
 
-	image = g_build_filename (THEME_DIR, theme_name, name, NULL);
-	
+        tmp = g_strdup_printf ("%s", theme_name);
+
+	image = g_build_filename (THEME_DIR_UNINSTALLED, tmp, name, NULL);
+        if (g_file_test (image,
+                         G_FILE_TEST_IS_REGULAR |
+                         G_FILE_TEST_IS_SYMLINK) == TRUE) {
+                g_free (tmp);
+                return image;
+        }
+
+	g_free (image);
+	image = g_build_filename (THEME_DIR, tmp, name, NULL);
+
+	g_free (tmp);
 	return image;
 }
 
@@ -803,6 +838,9 @@ cd_display_parse_theme (CDDisplay *disp,
 	CDDisplayPrivate *priv;
 	CDDisplayTheme *theme;
 
+	if (!disp)
+		return;
+
 	priv = disp->priv;
 	/* Should probably destroy the old theme here */
 
