@@ -148,7 +148,7 @@ struct _GSRWindowPrivate {
 };
 
 static gboolean            make_record_source      (GSRWindow         *window);
-static void                fill_record_input       (GSRWindow         *window);
+static void                fill_record_input       (GSRWindow         *window, gchar *selected);
 static GSRWindowPipeline * make_record_pipeline    (GSRWindow         *window);
 static GSRWindowPipeline * make_play_pipeline      (GSRWindow         *window);
 
@@ -673,8 +673,8 @@ do_save_file (GSRWindow *window,
 			return;
 	}
 
-	tmp = g_strdup_printf ("file://%s", name);
-	src = g_strdup_printf ("file://%s", priv->record_filename);
+	tmp = g_filename_to_uri (name, NULL, NULL);
+	src = g_filename_to_uri (priv->record_filename, NULL, NULL);
 	src_uri = gnome_vfs_uri_new (src);
 	dst_uri = gnome_vfs_uri_new (tmp);
 	g_free (src);
@@ -780,12 +780,11 @@ file_save_as_cb (GtkAction *action,
 	gtk_widget_destroy (file_chooser);
 }
 
-
 static void
 file_save_cb (GtkAction *action,
 	      GSRWindow *window)
 {
-	if (window->priv->dirty) {
+	if (!window->priv->has_file) {
 		file_save_as_cb (NULL, window);
 	} else {
 		do_save_file (window, window->priv->filename);
@@ -1213,7 +1212,7 @@ play_cb (GtkAction *action,
 {
 	GSRWindowPrivate *priv = window->priv;
 
-	if (priv->has_file == FALSE)
+	if (priv->has_file == FALSE && !priv->working_file)
 		return;
 
 	if (priv->play) {
@@ -1222,14 +1221,18 @@ play_cb (GtkAction *action,
 
 	if ((priv->play = make_play_pipeline (window))) {
 		gchar *uri;
+		gchar *usefile;
+
+		if(priv->has_file == FALSE && priv->working_file) usefile = priv->working_file;
+		else usefile = priv->filename;
 
 		/* FIXME: do we need to complete relative paths
 		 * here or is it always an absolute path? */
-		if (!g_path_is_absolute (priv->filename)) {
+		if (!g_path_is_absolute (usefile)) {
 			g_warning ("Filename '%s' is not an absolute path (FIXME)",
-			           priv->filename);
+			           usefile);
 		}
-		uri = g_strdup_printf ("file://%s", priv->filename);
+		uri = g_filename_to_uri (usefile, NULL, NULL);
 		g_object_set (window->priv->play->pipeline, "uri", uri, NULL);
 		g_free (uri);
 
@@ -1270,9 +1273,11 @@ record_cb (GtkAction *action,
 	GSRWindowPrivate *priv = window->priv;
 
 	if (priv->record) {
+		char *current_source;
 		shutdown_pipeline (priv->record);
 		if (!make_record_source (window)) exit (1);
-		fill_record_input (window);
+		current_source = gtk_combo_box_get_active_text (GTK_COMBO_BOX (window->priv->input));
+		fill_record_input (window, current_source);
 	}
 
 	if ((priv->record = make_record_pipeline (window))) {
@@ -1658,7 +1663,7 @@ record_start (gpointer user_data)
 	gchar *name;
 
 	window->priv->get_length_attempts = 16;
-	g_timeout_add (200, (GSourceFunc) record_tick_callback, window);
+	window->priv->tick_id = g_timeout_add (200, (GSourceFunc) record_tick_callback, window);
 
 	set_action_sensitive (window, "Stop", TRUE);
 	set_action_sensitive (window, "Play", FALSE);
@@ -1738,6 +1743,10 @@ record_state_changed_cb (GstBus *bus, GstMessage *msg, GSRWindow *window)
 		gtk_statusbar_push (GTK_STATUSBAR (window->priv->statusbar),
 				    window->priv->status_message_cid,
 				    _("Ready"));
+		if (window->priv->tick_id > 0) {
+			g_source_remove (window->priv->tick_id);
+			window->priv->tick_id = 0;
+		}
 		break;
 	default:
 		break;
@@ -1770,6 +1779,12 @@ make_record_source (GSRWindow *window)
 	}
 	window->priv->source = source;
 	e = gst_bin_get_by_interface (GST_BIN (source), GST_TYPE_MIXER);
+	if (GST_IS_MIXER (e) == FALSE) {
+		show_error_dialog (NULL, NULL,
+			_("Your audio capture settings are invalid. "
+			  "Please correct them in the Multimedia settings."));
+		return FALSE;
+	}
 	window->priv->mixer = GST_MIXER (e);
 
 	return TRUE;
@@ -1786,19 +1801,36 @@ record_input_changed_cb (GtkComboBox *input, GSRWindow *window)
 	text = gtk_combo_box_get_active_text (input);
 	GST_DEBUG ("record input changed to '%s'", GST_STR_NULL (text));
 
+	if (text == NULL)
+		return;
+
+	/* The pipeline has been destroyed already, we'll try and remember
+	 * the input for the next record run in fill_record_input() */
+	if (GST_IS_MIXER (window->priv->mixer) == FALSE)
+		return;
+
 	for (l = gst_mixer_list_tracks (window->priv->mixer);
 	     l != NULL; l = l->next) {
 		t = l->data;
+		if (t == NULL || t->label == NULL)
+			continue;
 		if ((strcmp (t->label, text) == 0) &&
 		    (t->flags & GST_MIXER_TRACK_INPUT)) {
 			if (new == NULL)
 				new = g_object_ref (t);
+		/* FIXME selected == t is equivalent to NULL == t in this case,
+		 * selected, after its initialization to NULL, was never written to
+		 * before this read access to it
+		 * and NULL == t is equivalent to FALSE, because of the check
+		 * "if (t == NULL || t->label == NULL)" above
+		 */
 		} else if (selected == t)
 			/* re-mute old one */
 			gst_mixer_set_record (window->priv->mixer,
 					      selected, FALSE);
 	}
 
+	/* FIXME selected _is_ NULL always at this point - same as 5 lines above*/
 	if (selected != NULL)
 		g_object_unref (selected);
 	if (!(selected = new))
@@ -1809,7 +1841,7 @@ record_input_changed_cb (GtkComboBox *input, GSRWindow *window)
 }
 
 static void
-fill_record_input (GSRWindow *window)
+fill_record_input (GSRWindow *window, gchar *selected)
 {
 	GstElement *e;
 	const GList *l;
@@ -1825,11 +1857,16 @@ fill_record_input (GSRWindow *window)
 
 	for (l = gst_mixer_list_tracks (window->priv->mixer); l != NULL; l = l->next) {
 		GstMixerTrack *t = l->data;
+		if (t->label == NULL)
+			continue;
 		if (t->flags & GST_MIXER_TRACK_INPUT) {
 			gtk_combo_box_append_text (GTK_COMBO_BOX (window->priv->input), t->label);
 			++i;
 		}
-		if (t->flags & GST_MIXER_TRACK_RECORD) {
+		if ((t->flags & GST_MIXER_TRACK_RECORD) && (selected == NULL)) {
+			gtk_combo_box_set_active (GTK_COMBO_BOX (window->priv->input), i - 1);
+		}
+		if ((selected != NULL) && strcmp (selected, t->label) == 0) {
 			gtk_combo_box_set_active (GTK_COMBO_BOX (window->priv->input), i - 1);
 		}
 	}
@@ -2151,6 +2188,7 @@ gsr_window_init (GSRWindow *window)
 	gtk_widget_show (menubar);
 
 	toolbar = gtk_ui_manager_get_widget (priv->ui_manager, "/ToolBar");
+	gtk_toolbar_set_show_arrow (GTK_TOOLBAR (toolbar), FALSE);
 	gtk_box_pack_start (GTK_BOX (main_vbox), toolbar, FALSE, FALSE, 0);
 	gtk_widget_show (toolbar);
 
@@ -2202,10 +2240,14 @@ gsr_window_init (GSRWindow *window)
 	gtk_widget_show (priv->input);
 
 	if (!make_record_source (window)) exit (1);
-	fill_record_input (window);
+	fill_record_input (window, NULL);
 	g_signal_connect (priv->input, "changed",
 			  G_CALLBACK (record_input_changed_cb), window);
 
+	atk_object_add_relationship (gtk_widget_get_accessible (GTK_WIDGET (priv->input)),
+				ATK_RELATION_LABELLED_BY,
+				gtk_widget_get_accessible (GTK_WIDGET (label)));
+
 	/* choose profile */
 	hbox = gtk_hbox_new (FALSE, 12);
 	gtk_box_pack_start (GTK_BOX (content_vbox), hbox, FALSE, FALSE, 0);
@@ -2218,6 +2260,10 @@ gsr_window_init (GSRWindow *window)
 	gtk_box_pack_start (GTK_BOX (hbox), window->priv->profile, TRUE, TRUE, 0);
 	gtk_widget_show (window->priv->profile);
 
+	atk_object_add_relationship (gtk_widget_get_accessible (GTK_WIDGET (priv->profile)),
+				ATK_RELATION_LABELLED_BY,
+				gtk_widget_get_accessible (GTK_WIDGET (label)));
+
 	id = gconf_client_get_string (gconf_client, KEY_LAST_PROFILE_ID, NULL);
 	if (id) {
 		gm_audio_profile_choose_set_active (window->priv->profile, id);
@@ -2282,6 +2328,7 @@ gsr_window_init (GSRWindow *window)
 
 	/* statusbar */
 	priv->statusbar = gtk_statusbar_new ();
+	GTK_WIDGET_SET_FLAGS (priv->statusbar, GTK_CAN_FOCUS);
 	gtk_box_pack_end (GTK_BOX (main_vbox), priv->statusbar, FALSE, FALSE, 0);
 	gtk_widget_show (priv->statusbar);
 	priv->status_message_cid = gtk_statusbar_get_context_id
@@ -2504,7 +2551,7 @@ GtkWidget *
 gsr_window_new (const char *filename)
 {
 	GSRWindow *window;
-	struct stat buf;
+	char *template;
 
 	/* filename has been changed to be without extension */
 	window = g_object_new (GSR_TYPE_WINDOW, 
@@ -2512,15 +2559,15 @@ gsr_window_new (const char *filename)
 			       NULL);
         /* FIXME: check extension too */
 	window->priv->filename = g_strdup (filename);
-	if (stat (filename, &buf) == 0) {
+	if (g_file_test (filename, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_REGULAR) != FALSE) {
 		window->priv->has_file = TRUE;
 	} else {
 		window->priv->has_file = FALSE;
 	}
 
-	window->priv->record_filename = g_strdup_printf ("%s/gsr-record-%s-%d.XXXXXX",
-							 g_get_tmp_dir(), filename, getpid ());
-	window->priv->record_fd = g_mkstemp (window->priv->record_filename);
+	template = g_strdup_printf ("gsr-record-%s-%d.XXXXXX", filename, getpid ());
+	window->priv->record_fd = g_file_open_tmp (template, &window->priv->record_filename, NULL);
+	g_free (template);
 	close (window->priv->record_fd);
 
 	if (window->priv->has_file == FALSE) {
