@@ -20,7 +20,6 @@
  */
 
 #include "config.h"
-#include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include <glade/glade.h>
 
@@ -28,10 +27,15 @@ GladeXML*
 gmp_util_load_glade_file (const char *filename,
                           const char *widget_root,
                           GtkWindow  *error_dialog_parent);
+
 void
 gmp_util_show_error_dialog (GtkWindow *transient_parent,
                             GtkWidget **weak_ptr,
-                            const char *message_format, ...);
+                            const char *message_format, ...) G_GNUC_PRINTF (3, 4);
+
+void
+gmp_util_run_error_dialog (GtkWindow *transient_parent,
+                           const char *message_format, ...);
 
 #ifdef DEBUG
 #define GMP_DEBUG(...) G_STMT_START{ \
